from pulp import *
import sys

N = int(sys.argv[1])
MAX_THROWS = int(sys.argv[2])

possibilities = {str(i)+str(j): 1./N**2 if i==j else 2./N**2 for i in range(1,N+1) for j in range(1,i+1)}
ordering = [str(i)+str(j) for i in range(1,N+1) for j in range(1,i) if i!=2 or j!=1] + \
           [str(i)*2 for i in range(1,N+1)] + ['21']

def bigger(a,b):
  """ Bigger or equal """
  return b not in ordering or a==max(a, b, key=ordering.index)

cache = {}

def getOptimalMax(lastSaid, player, max_throws):
#  print lastSaid, player, max_throws
  def getOptimal(lastSaid_, player_):
    if lastSaid_ == lastSaid:
      return getOptimalMax(lastSaid_, player_, max(max_throws-1, 0) )
    else:
      return getOptimalMax(lastSaid_, player_, MAX_THROWS)

  if (lastSaid, player, max_throws) not in cache:
    prob = LpProblem("Machacek"+lastSaid+str(player), LpMaximize)

    # Variables
    lp = {}
    lp['q'] = LpVariable('q')
    prob += lp['q'] # = objective

    if player == 0: # I am on the move
      if lastSaid == '21' and max_throws == 0:
        cache[(lastSaid, player, max_throws)] = -1.0 # because other player said 21 and I believed 
        return -1.0
      else:
        toQreject = {}
        toQaccept = {}
        for throw, p in possibilities.items():
          total = []
          for say in ordering:
            if bigger(say, lastSaid) and (max_throws != 0 or say!=lastSaid):
              xname = 'xt' + throw + 's' + say
              qname = 'qs' + say
              lp[xname] = LpVariable(xname, 0)
              xname_q = lp[xname] if throw==say else -lp[xname]
              toQreject[qname] = toQreject.setdefault(qname, []) + [xname_q]
              toQaccept[qname] = toQaccept.setdefault(qname, []) + [getOptimal(say,1) * lp[xname]]
              total.append((1/p)*lp[xname])
          prob += lpSum(total) == 1
        for qname in toQreject:
          lp[qname] = LpVariable(qname)
          prob += lpSum(toQreject[qname]) >= lp[qname] # for rejecting
          prob += lpSum(toQaccept[qname]) >= lp[qname] # for accepting
        prob += lpSum([lp[qname] for qname in toQreject]) >= lp['q']
    else: # enemy on move
      if lastSaid == '21' and max_throws == 0:
        cache[(lastSaid, player, max_throws)] = 1.0 # because I said 21 and the enemy accepted
        return 1.0
      else:
        total = []
        for say in ordering:
          if bigger(say, lastSaid) and (max_throws != 0 or say!=lastSaid):
            xname = 'xS' + say
            xnamex = xname + 'x'
            lp[xname] = LpVariable(xname, 0)
            lp[xnamex] = LpVariable(xnamex, 0)
            prob += lp[xname] + lp[xnamex] == 1
        for throw, p in possibilities.items():
          qname = 'qT' + throw
          lp[qname] = LpVariable(qname)
          total.append(p*lp[qname])
          reject_total = []
          for say in ordering:
            if bigger(say, lastSaid) and (max_throws != 0 or say!=lastSaid):
              xname = 'xS' + say
              xnamex = xname + 'x'
              c = 1 if say!=throw else -1
              prob += getOptimal(say, 0)*lp[xname] + c*lp[xnamex] >= lp[qname]
              pass
        prob += lpSum(total) >= lp['q']

    prob.solve(GLPK(msg=0))

    if max_throws == MAX_THROWS:
      print
      print "Strategy for:" , lastSaid, "player:", player
      # Solution
      for v in prob.variables():
        if v.varValue != 0:
          print v.name, "=", v.varValue

      print "objective=", value(prob.objective), "for last said", lastSaid, "player", player
    cache[(lastSaid, player, max_throws)] = value(prob.objective)

  return cache[(lastSaid, player, max_throws)] 

for thr in ordering[::-1]:
  getOptimalMax(thr, 0, MAX_THROWS)
  getOptimalMax(thr, 1, MAX_THROWS)

getOptimalMax('--', 0, MAX_THROWS)
